;
; CHAR.ASM
; Characters (sprites) handling
;

; CHARACTER_INIT() ----------------------------------------------------------
; Initialize character variables
;----------------------------------------------------------------------------
character_init:
        xor     a
        ld      (VAR_character_current), a
        ret
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
character_infos_table:
.define MARIO_IDX 51

; Japan My-Cards Characters
.dw     data_teddyboy_infos
.db     2
.dw     data_pitpot_infos
.db     4
.dw     data_myhero_infos
.db     3
.dw     data_mgunjoe_infos
.db     6
.dw     data_gsthouse_infos
.db     2
.dw     data_spyvsspy_infos
.db     3

; Japan Cartridges Characters
.dw     data_fntzone1_infos
.db     6
.dw     data_prowrest_infos
.db     5
.dw     data_hokutono_infos
.db     4
.dw     data_akiddmw_infos
.db     2
.dw     data_theninja_infos
.db     2
.dw     data_dtarget_infos
.db     4
.dw     data_wboy1_infos
.db     3
.dw     data_gbasket_infos
.db     4

.dw     data_kunfukid_infos
.db     2
.dw     data_zillion_infos
.db     3
.dw     data_wsoccer_infos
.db     6
.dw     data_anmitsuh_infos
.db     2
.dw     data_fntzone2_infos
.db     4
.dw     data_pengland_infos
.db     2
.dw     data_aztecadv_infos
.db     2
.dw     data_aliensyn_infos
.db     6

.dw     data_phanstar_infos
.db     3
.dw     data_zillion2_infos
.db     2
.dw     data_wboy2_infos
.db     5
.dw     data_akiddls_infos
.db     5
.dw     data_shinobi_infos
.db     3
.dw     data_bakabon_infos
.db     2
.dw     data_csilver_infos
.db     4
.dw     data_kenseidn_infos
.db     4

.dw     data_lordswrd_infos
.db     5
.dw     data_bublbobl_infos
.db     4
.dw     data_golvlius_infos
.db     5
.dw     data_kujakuou_infos
.db     6
.dw     data_ddragon_infos
.db     6
.dw     data_ys_infos
.db     3
.dw     data_borgman_infos
.db     3
.dw     data_rygar_infos
.db     5
.dw     data_solomonk_infos
.db     6


; World Cartridges Characters
.dw     data_rastan_infos       ; 1988
.db     2
.dw     data_montezum_infos     ; 1989
.db     4
.dw     data_dyndux_infos       ; 1989
.db     5
.dw     data_psychofx_infos     ; 1989
.db     2
.dw     data_psychof2_infos     ; 1989
.db     4
.dw     data_wboy3_infos        ; 1989
.db     2
.dw     data_akiddsw_infos      ; 1990
.db     6
.dw     data_asterix_infos      ; 1991
.db     5
.dw     data_sonic_infos        ; 1991
.db     5
.dw     data_ngaiden_infos      ; 1992
.db     5

; Misc Characters
.dw     data_snail_infos        ; 1986
.db     4
.dw     data_mario_infos
.db     6

.dw     0
;----------------------------------------------------------------------------

; CHARACTER_NEXT() ----------------------------------------------------------
; Get and initialize next character
;----------------------------------------------------------------------------
character_next:
        xor     a
        ld      (VAR_character_anim_frame_cur), a
        ld      (VAR_character_anim_frame_max), a
        ld      a, 8
        ld      (VAR_character_anim_delay), a

character_next_rewind:
        ld      hl, character_infos_table
        ld      a, (VAR_character_current)
        inc     a
        ld      (VAR_character_current), a
        dec     a

        ld      b, a    ; A *= 3
        rlc     a       ;
        add     a, b    ;

        ld      d, 0
        ld      e, a
        add     hl, de
        ld      a, (hl)
        ld      (VAR_character_infos+0), a
        inc     hl
        ld      a, (hl)
        ld      b, a
        ld      (VAR_character_infos+1), a

        or      b
        jr      nz, character_copy
        xor     a
        ld      (VAR_character_current), a
        jr      character_next_rewind
;----------------------------------------------------------------------------

; CHARACTER_COPY() ----------------------------------------------------------
; Setup infos, palette and tiles for current character
;----------------------------------------------------------------------------
character_copy:

        ; Setup Mapper to access this character
        inc     hl
        ld      a, (hl)
        ld      (REG_MAP_2), a

        ld      hl, (VAR_character_infos)

        ld      a, (hl)
        rlc     a
        rlc     a
        rlc     a
        ld      b, a
        ld      a, -4
        sub     b
        ld      (VAR_character_pos_x), a

        inc     hl                      ; Skip tiles x
        inc     hl                      ; Skip tiles y

        ld      a, 191-2*8              ; Calculate vertical position
        sub     (hl)                    ;
        ld      (VAR_character_pos_y), a
        inc     hl                      ; Skip pixels y

        inc     hl                      ; Skip number of tiles
        inc     hl                      ; Skip number of data frames

        ld      a, (hl)
        ld      (VAR_character_data_size_total), a
        inc     hl
        ld      a, (hl)
        ld      (VAR_character_data_size_total+1), a
        inc     hl
        ld      a, (hl)
        ld      (VAR_character_anim_frame_max), a
        inc     hl
        ld      a, (hl)
        inc     hl
        ld      (VAR_character_anim_delay), a
        ld      (VAR_character_anim_delay_frame), a
        ld      a, (VAR_character_anim_frame_max)
        ld      d, 0
        ld      e, a
        add     hl, de

        ; Copy Description
        ld      de, VRAM_BG_MAP + (1 + 8 * 32) * 2
        ld      c,  0
        call    vdp_print
        inc     hl

        ; Copy Palette (skip color 0)
        inc     hl
        ld      a, 17
        ld      b, 15
        call    vdp_set_pal

        ; Copy Tiles
        ld      bc, (VAR_character_data_size_total)
        ld      de, $0000 + VRAM_TILE_SIZE * 256
        call    vdp_load_data

        ret
;----------------------------------------------------------------------------

; CHARACTER_REFRESH() -------------------------------------------------------
; Refresh character sprites
;----------------------------------------------------------------------------
character_refresh:

        ld      hl, (VAR_character_infos)

        ld      a, (hl)                 ; Load size_x in C
        ld      c, a                    ;
        inc     hl                      ;

        ld      a, (hl)                 ; Load size_y in B
        rrc     a                       ;
        ld      b, a                    ;
        inc     hl                      ;

        ld      a, (VAR_character_pos_y); Load vertical starting position
        inc     hl                      ; Skip vertical size in pixel

        push    bc

        push    af                      ;
        ld      de, VRAM_SPR_MAP        ; Set Sprite Y map address
        rst     $18                     ;
        pop     af                      ;

        ld      d, c
character_refresh_map_y_loop:
        out     (VDP_DATA), a
        dec     c
        jr      nz, character_refresh_map_y_loop
        add     a, 16
        ld      c, d
        djnz    character_refresh_map_y_loop

        ld      a, VRAM_SPR_LAST        ; Write end-of-sprite marker
        out     (VDP_DATA), a           ;

        pop     bc
        dec     sp
        dec     sp

        ld      de, VRAM_SPR_MAP+128    ; Set Sprite X/N map address
        rst     $18                     ;

        ld      a, (hl)                 ; Load number of tiles per frame into E
        ld      e, a                    ;
        inc     hl                      ;

        inc     hl                      ; Skip number of data frames
        inc     hl                      ; Skip number of bytes
        inc     hl                      ;
        inc     hl                      ; Skip number of animation frames
        inc     hl                      ; Skip animation speed

        ld      a, (VAR_character_anim_frame_cur) ; Load current frame into D
        push    de
        ld      d, 0
        ld      e, a
        add     hl, de
        pop     de
        ld      a, (hl)
        ld      d, a
        xor     a
character_refresh_map_n_compute_loop:
        add     a, e
        dec     d
        jr      nz, character_refresh_map_n_compute_loop

        ld      d, c                    ; D = Backup of C
        ld      e, a                    ; E = N (char)

        ld      a, (VAR_character_pos_x); A = X (pos)
character_refresh_map_xn_loop:
        out     (VDP_DATA), a
        push    af
        ld      a, e
        out     (VDP_DATA), a
        pop     af
        inc     e
        inc     e
        add     a, 8
        dec     c
        jr      nz, character_refresh_map_xn_loop
        ld      a, (VAR_character_pos_x); A = X
        ld      c, d
        djnz    character_refresh_map_xn_loop

        pop     bc

        ret
;----------------------------------------------------------------------------

; CHARACTER_UPDATE() --------------------------------------------------------
; Update character (move, open door, change..)
;----------------------------------------------------------------------------
character_update:
        push    af
        push    bc
        push    de
        push    hl

        ; POSITION
character_update_move:
        ld      a, (VAR_character_pos_x)
        inc     a
        ld      (VAR_character_pos_x), a

        ; DOOR REINIT TEST
        ld      a, (VAR_mario_kicking)
        and     a
        jr      nz, +9
        ld      a, (VAR_character_pos_x)
        and     a
        jr      nz, +3
        call    door_reinit

        ; DOOR TEST
        ld      hl, (VAR_character_infos)
        ld      a, (hl)
        rlc     a
        rlc     a
        ld      b, a
        ld      a, (VAR_character_pos_x)
        add     a, b
        cp      180+16+8
        jr      nz, character_update_opendoor_end
character_update_opendoor:
        ld      a, 256-3*16+8
        ld      (VAR_character_pos_x), a
        call    door_open
        call    vdp_disable_sprites
        ; call    character_next
        jp      character_update_end
character_update_opendoor_end:

        ; ANIMATION
        ld      a, (VAR_character_anim_delay)
        dec     a
        ld      (VAR_character_anim_delay), a
        jr      nz, character_update_anim_end
        ld      a, (VAR_character_anim_delay_frame)
        ld      (VAR_character_anim_delay), a
character_update_anim:
        ld      a, (VAR_character_anim_frame_max)
        ld      b, a
        ld      a, (VAR_character_anim_frame_cur)
        inc     a
        cp      b
        jr      nz, +1
        xor     a
        ld      (VAR_character_anim_frame_cur), a
character_update_anim_end:

        ; REFRESH
        call    character_refresh

character_update_end:
        pop     hl
        pop     de
        pop     bc
        pop     af
        ret
;----------------------------------------------------------------------------

